@@ -64,13 +64,14 @@ from bpy.props import (
 
 # import gRPC
 import grpc
+import toxicblend
 import toxicblend.toxicblend_pb2_grpc as toxicblend_pb2_grpc
 import toxicblend.toxicblend_pb2 as toxicblend_pb2
 
 bl_info = {
     "name": "Toxicblend MeshTools",
     "author": "EADF",
-    "version": (0, 0, 8),
+    "version": (0, 0, 13),
     "blender": (2, 92, 0),
     "location": "View3D > Sidebar > Edit Tab / Edit Mode Context Menu",
     "warning": "Communicates with a gRPC server on localhost",
@@ -81,6 +82,9 @@ bl_info = {
 
 SERVER_URL = 'localhost:50069'
 
+def check_toxicblend_version():
+    if toxicblend.__version__ != '0.2.0':
+        raise ToxicblendException("pip package toxicblend version 0.2.0 is required")
 
 class ToxicblendException(Exception):
     def __init__(self, message):
@@ -240,10 +244,14 @@ def handle_received_object(active_object, pb_message, remove_doubles_threshold=N
         if option.key == "PACKED_FACES" and option.value == "True":
             packed_faces = True
 
-    # active_object.select = False
+    if len(pb_message.models) > 0:
+       pb_model = pb_message.models[0]
+    elif len(pb_message.models32) > 0:
+       pb_model = pb_message.models32[0]
+    else:
+       raise ToxicblendException("No return models found")
 
-    if len(pb_message.models) == 1:
-        pb_model = pb_message.models[0]
+    if not pb_model is None:
         (vertices, edges, faces, matrix) = get_pydata(pb_model, only_edges, packed_faces)
         if len(faces) > 0 or len(edges) > 0:
             new_mesh = bpy.data.meshes.new(pb_model.name)
@@ -350,6 +358,8 @@ class ToxicBlend_debug_object(bpy.types.Operator):
         return context.active_object is not None
 
     def execute(self, context):
+
+        check_toxicblend_version()
         problem_found = False
         active_object = context.view_layer.objects.active
         bm = bmesh.new()
@@ -415,6 +425,7 @@ class ToxicBlend_SelectEndVertices(Operator):
 
     def execute(self, context):
 
+        check_toxicblend_version()
         # Get the active mesh
         obj = bpy.context.edit_object
         me = obj.data
@@ -465,6 +476,7 @@ class ToxicBlend_SelectCollinearEdges(Operator):
 
     def execute(self, context):
 
+        check_toxicblend_version()
         # Get the active mesh
         obj = bpy.context.edit_object
         me = obj.data
@@ -510,6 +522,7 @@ class ToxicBlend_SelectCollinearEdges(Operator):
 
                 # print("from_v:", from_v, " is connected to edge :", candidate_e.index)
                 if to_v == candidate_e.verts[0].index:
+                    # todo: use vertex.calc_edge_angle()
                     angle = angle_between_edges(bm.verts[from_v].co, bm.verts[to_v].co,
                                                 bm.verts[candidate_e.verts[1].index].co)
                     # print("from_v<->to_v<->candidate_e.verts[1] angle:", angle)
@@ -518,6 +531,7 @@ class ToxicBlend_SelectCollinearEdges(Operator):
                         work_queue.add(candidate_e)
 
                 elif to_v == candidate_e.verts[1].index:
+                    # todo: use vertex.calc_edge_angle()
                     angle = angle_between_edges(bm.verts[from_v].co, bm.verts[to_v].co,
                                                 bm.verts[candidate_e.verts[0].index].co)
                     # print("from_v<->to_v<->candidate_e.verts[0] angle:",angle)
@@ -535,6 +549,7 @@ class ToxicBlend_SelectCollinearEdges(Operator):
 
                 # print("from_v:", from_v, " is connected to edge :", candidate_e.index)
                 if to_v == candidate_e.verts[0].index:
+                    # todo: use vertex.calc_edge_angle()
                     angle = angle_between_edges(bm.verts[from_v].co, bm.verts[to_v].co,
                                                 bm.verts[candidate_e.verts[1].index].co)
                     # print("from_v<->to_v<->candidate_e.verts[1] angle:", angle)
@@ -543,6 +558,7 @@ class ToxicBlend_SelectCollinearEdges(Operator):
                         work_queue.add(candidate_e)
 
                 elif to_v == candidate_e.verts[1].index:
+                    # todo: use vertex.calc_edge_angle()
                     angle = angle_between_edges(bm.verts[from_v].co, bm.verts[to_v].co,
                                                 bm.verts[candidate_e.verts[0].index].co)
                     # print("from_v<->to_v<->candidate_e.verts[0] angle:", angle)
@@ -573,6 +589,7 @@ class ToxicBlend_SelectIntersectionVertices(Operator):
 
     def execute(self, context):
 
+        check_toxicblend_version()
         # Get the active mesh
         obj = bpy.context.edit_object
         me = obj.data
@@ -600,11 +617,65 @@ class ToxicBlend_SelectIntersectionVertices(Operator):
         return {'FINISHED'}
 
 
+class ToxicBlend_SelectVerticesUntilIntersection(Operator):
+    """Selects all (wire-frame) vertices that are connected to already selected vertices until an intersection is detected (offline plugin)"""
+    bl_idname = "mesh.toxicblend_meshtools_select_vertices_until_intersection"
+    bl_label = "Select vertices until intersection"
+    bl_description = "Selects all (wire-frame) vertices that are connected to already selected vertices until an intersection is detected (offline plugin)"
+    bl_options = {'REGISTER', 'UNDO'}  # enable undo for the operator.
+
+    @classmethod
+    def poll(cls, context):
+        return context.active_object is not None
+
+    def execute(self, context):
+
+        check_toxicblend_version()
+        # Get the active mesh
+        obj = bpy.context.edit_object
+        me = obj.data
+
+        # Get a BMesh representation
+        bm = bmesh.from_edit_mesh(me)
+        bm.verts.ensure_lookup_table()
+        bm.edges.ensure_lookup_table()
+        bm.faces.ensure_lookup_table()
+
+        if len(bm.edges) > 0 and len(bm.faces) == 0:
+
+            already_selected = set()  # key by vertex.index
+            work_queue = set()  # vertex.index
+
+            for v in bm.verts:
+                if v.select:
+                    work_queue.add(v.index)
+
+            while len(work_queue) > 0:
+                v = work_queue.pop()
+                if v in already_selected:
+                    continue
+
+                if len(bm.verts[v].link_edges) <= 2:
+                    bm.verts[v].select = True
+                    for e in bm.verts[v].link_edges:
+                        if e.verts[0].index != v and e.verts[0].index not in already_selected:
+                            work_queue.add(e.verts[0].index)
+                        if e.verts[1].index != v and e.verts[1].index not in already_selected:
+                            work_queue.add(e.verts[1].index)
+
+                # only mark vertices as already_selected if they've been through the loop once
+                already_selected.add(v)
+
+        # Show the updates in the viewport
+        bmesh.update_edit_mesh(me, False)
+        return {'FINISHED'}
+
+
 # 2d_outline operator
 class Toxicblend_2D_Outline(Operator):
     bl_idname = "mesh.toxicblend_meshtools_2d_outline"
     bl_label = "2D Outline"
-    bl_description = "Outline 2d geometry, the geometry must be flat and on a plane intersecting origin"
+    bl_description = "Outline 2d geometry into a wire frame, the geometry must be flat and on a plane intersecting origin"
     bl_options = {'REGISTER', 'UNDO'}
 
     @classmethod
@@ -618,6 +689,8 @@ class Toxicblend_2D_Outline(Operator):
         return self.execute(context)
 
     def execute(self, context):
+
+        check_toxicblend_version()
         # initialise
         active_object, active_mesh = initialise()
         settings_write(self)
@@ -629,8 +702,8 @@ class Toxicblend_2D_Outline(Operator):
 
                 response = stub.execute(command)
                 handle_response(response)
-                if len(response.models) > 0:
-                    print("client received options: ", len(response.options), " models:", len(response.models))
+                if len(response.models) > 0 or len(response.models32) > 0:
+                    print("client received options: ", len(response.options), " models64:", len(response.models), " models32:", len(response.models32))
                     handle_received_object(active_object, response)
 
             # cleaning up
@@ -660,7 +733,7 @@ class Toxicblend_Voronoi(Operator):
 
     distance: FloatProperty(
         name="Discretization Distance",
-        description="Discretization distance as a percentage of the larges axis, used in the discretization process of curved edges",
+        description="Discrete distance as a percentage of the AABB. This value is used when sampling parabolic arc edges.",
         default=0.001,
         min=0.000101,
         max=0.999999,
@@ -685,6 +758,8 @@ class Toxicblend_Voronoi(Operator):
         return self.execute(context)
 
     def execute(self, context):
+
+        check_toxicblend_version()
         # initialise
         active_object, active_mesh = initialise()
         settings_write(self)
@@ -708,8 +783,8 @@ class Toxicblend_Voronoi(Operator):
                 build_pb_model(active_object, active_mesh, command.models.add())
                 response = stub.execute(command)
                 handle_response(response)
-                if len(response.models) > 0:
-                    print("client received options: ", len(response.options), " models:", len(response.models))
+                if len(response.models) > 0 or len(response.models32) > 0:
+                    print("client received options: ", len(response.options), " models64:", len(response.models), " models32:", len(response.models32))
                     handle_received_object(active_object, response)
 
             # cleaning up
@@ -742,6 +817,8 @@ class Toxicblend_Knife_Intersect(Operator):
         return self.execute(context)
 
     def execute(self, context):
+
+        check_toxicblend_version()
         # initialise
         active_object, active_mesh = initialise()
         settings_write(self)
@@ -753,8 +830,8 @@ class Toxicblend_Knife_Intersect(Operator):
 
                 response = stub.execute(command)
                 handle_response(response)
-                if len(response.models) > 0:
-                    print("client received options: ", len(response.options), " models:", len(response.models))
+                if len(response.models) > 0 or len(response.models32) > 0:
+                    print("client received options: ", len(response.options), " models64:", len(response.models), " models32:", len(response.models32))
                     handle_received_object(active_object, response)
 
             # cleaning up
@@ -800,7 +877,7 @@ class Toxicblend_Centerline(Operator):
 
     distance: FloatProperty(
         name="Distance",
-        description="Discrete distance as a percentage of the AABB",
+        description="Discrete distance as a percentage of the AABB. This value is used when sampling parabolic arc edges. It is also used for simplification.",
         default=0.005,
         min=0.0001,
         max=4.9999,
@@ -825,6 +902,8 @@ class Toxicblend_Centerline(Operator):
         return self.execute(context)
 
     def execute(self, context):
+
+        check_toxicblend_version()
         # initialise
         active_object, active_mesh = initialise()
         settings_write(self)
@@ -851,8 +930,8 @@ class Toxicblend_Centerline(Operator):
 
                 response = stub.execute(command)
                 handle_response(response)
-                if len(response.models) > 0:
-                    print("client received options: ", len(response.options), " models:", len(response.models))
+                if len(response.models) > 0 or len(response.models32) > 0:
+                    print("client received options: ", len(response.options), " models64:", len(response.models), " models32:", len(response.models32))
                     handle_received_object(active_object, response)
 
             # cleaning up
@@ -876,7 +955,7 @@ class Toxicblend_Voronoi_Mesh(Operator):
 
     distance: FloatProperty(
         name="Distance",
-        description="Discrete distance as a percentage of the AABB",
+        description="Discrete distance as a percentage of the AABB. This value is used when sampling parabolic arc edges.",
         default=0.005,
         min=0.0001,
         max=4.9999,
@@ -895,6 +974,8 @@ class Toxicblend_Voronoi_Mesh(Operator):
         return self.execute(context)
 
     def execute(self, context):
+
+        check_toxicblend_version()
         # initialise
         active_object, active_mesh = initialise()
         settings_write(self)
@@ -910,8 +991,8 @@ class Toxicblend_Voronoi_Mesh(Operator):
 
                 response = stub.execute(command)
                 handle_response(response)
-                if len(response.models) > 0:
-                    print("client received options: ", len(response.options), " models:", len(response.models))
+                if len(response.models) > 0 or len(response.models32) > 0:
+                    print("client received options: ", len(response.options), " models64:", len(response.models), " models32:", len(response.models32))
                     handle_received_object(active_object, response)
 
             # cleaning up
@@ -945,7 +1026,7 @@ class Toxicblend_Voxel(Operator):
 
     divisions: FloatProperty(
         name="Voxel Divisions",
-        description="The larges scale of the AABB will be divided up into this number of voxels",
+        description="The longest axis of the model will be divided up into this number of voxels, the other axes will have proportionally number of voxels",
         default=100.0,
         min=50,
         max=400,
@@ -964,6 +1045,8 @@ class Toxicblend_Voxel(Operator):
         return self.execute(context)
 
     def execute(self, context):
+
+        check_toxicblend_version()
         # initialise
         active_object, active_mesh = initialise()
         settings_write(self)
@@ -973,7 +1056,7 @@ class Toxicblend_Voxel(Operator):
             with grpc.insecure_channel(SERVER_URL, options=channel_opt) as channel:
                 stub = toxicblend_pb2_grpc.ToxicBlendServiceStub(channel)
                 command = toxicblend_pb2.Command(command='voxel')
-                build_pb_model(active_object, active_mesh, command.models.add())
+                build_pb_model(active_object, active_mesh, command.models32.add())
 
                 opt = command.options.add()
                 opt.key = "RADIUS"
@@ -985,8 +1068,8 @@ class Toxicblend_Voxel(Operator):
 
                 response = stub.execute(command)
                 handle_response(response)
-                if len(response.models) > 0:
-                    print("client received options: ", len(response.options), " models:", len(response.models))
+                if len(response.models) > 0 or len(response.models32) > 0:
+                    print("client received options: ", len(response.options), " models64:", len(response.models), " models32:", len(response.models32))
                     handle_received_object(active_object, response)
 
             # cleaning up
@@ -1010,7 +1093,7 @@ class Toxicblend_Simplify(Operator):
 
     distance: FloatProperty(
         name="Distance",
-        description="Simplification distance",
+        description="Discrete distance as a percentage of the AABB. This value is used as a parameter to the Ramer–Douglas-Peucker line simplification algorithm.",
         default=0.01,
         min=0.00000001,
         max=4.99999999,
@@ -1029,6 +1112,8 @@ class Toxicblend_Simplify(Operator):
         return self.execute(context)
 
     def execute(self, context):
+
+        check_toxicblend_version()
         # initialise
         active_object, active_mesh = initialise()
         settings_write(self)
@@ -1043,8 +1128,8 @@ class Toxicblend_Simplify(Operator):
 
                 response = stub.execute(command)
                 handle_response(response)
-                if len(response.models) > 0:
-                    print("client received options: ", len(response.options), " models:", len(response.models))
+                if len(response.models) > 0 or len(response.models32) > 0:
+                    print("client received options: ", len(response.options), " models64:", len(response.models), " models32:", len(response.models32))
                     handle_received_object(active_object, response)
 
             # cleaning up
@@ -1078,6 +1163,7 @@ class VIEW3D_MT_edit_mesh_toxicblend_meshtools(Menu):
         layout.operator("mesh.toxicblend_meshtools_voxel")
         layout.operator("mesh.toxicblend_meshtools_select_end_vertices")
         layout.operator("mesh.toxicblend_meshtools_select_collinear_edges")
+        layout.operator("mesh.toxicblend_meshtools_select_vertices_until_intersection")
         layout.operator("mesh.toxicblend_meshtools_select_intersection_vertices")
         layout.operator("mesh.toxicblend_meshtools_debug_object")
 
@@ -1107,7 +1193,7 @@ class TB_MeshToolsProps(PropertyGroup):
     # general display properties
     simplify_distance: FloatProperty(
         name="Distance",
-        description="Simplification distance",
+        description="Discrete distance as a percentage of the AABB. This value is used as a parameter to the Ramer–Douglas-Peucker line simplification algorithm.",
         default=0.01,
         min=0.0,
         max=5.0,
@@ -1135,7 +1221,7 @@ class TB_MeshToolsProps(PropertyGroup):
 
     centerline_distance: FloatProperty(
         name="Distance",
-        description="Discrete distance as a percentage of the AABB",
+        description="Discrete distance as a percentage of the AABB. This value is used when sampling parabolic arc edges. It is also used for simplification.",
         default=0.01,
         min=0.005,
         max=99.0,
@@ -1157,7 +1243,7 @@ class TB_MeshToolsProps(PropertyGroup):
 
     voronoi_mesh_distance: FloatProperty(
         name="Distance",
-        description="Discrete distance as a percentage of the AABB",
+        description="Discrete distance as a percentage of the AABB. This value is used when sampling parabolic arc edges.",
         default=0.005,
         min=0.0001,
         max=4.9999,
@@ -1173,7 +1259,7 @@ class TB_MeshToolsProps(PropertyGroup):
 
     voronoi_distance: FloatProperty(
         name="Discretization Distance",
-        description="Discretization distance as a percentage of the larges axis, used in the discretization process of curved edges",
+        description="Discrete distance as a percentage of the AABB. This value is used when sampling parabolic arc edges.",
         default=0.001,
         min=0.000101,
         max=0.999999,
@@ -1209,7 +1295,7 @@ class TB_MeshToolsProps(PropertyGroup):
 
     voxel_divisions: FloatProperty(
         name="Voxel Divisions",
-        description="The larges scale of the AABB will be divided up into this number of voxels",
+        description="The longest axis of the model will be divided up into this number of voxels, the other axes will have proportionally number of voxels",
         default=100.0,
         min=50,
         max=400,
@@ -1237,6 +1323,7 @@ classes = (
     Toxicblend_Voxel,
     ToxicBlend_SelectEndVertices,
     ToxicBlend_SelectIntersectionVertices,
+    ToxicBlend_SelectVerticesUntilIntersection,
     ToxicBlend_SelectCollinearEdges,
     ToxicBlend_debug_object,
 )
