@@ -5,6 +5,7 @@ use crate::toxicblend_pb::KeyValuePair as PB_KeyValuePair;
 use crate::toxicblend_pb::Model as PB_Model;
 use crate::toxicblend_pb::Reply as PB_Reply;
 use crate::toxicblend_pb::Vertex as PB_Vertex;
+use boostvoronoi::geometry;
 use cgmath::{Angle, EuclideanSpace, SquareMatrix};
 use cgmath::{Transform, UlpsEq};
 use itertools::Itertools;
@@ -39,7 +40,7 @@ pub fn parse_input(
     input_pb_model: &PB_Model,
 ) -> Result<
     (
-        fnv::FnvHashSet<(usize, usize)>,
+        ahash::AHashSet<(usize, usize)>,
         Vec<cgmath::Point3<f64>>,
         cgmath_3d::Aabb3<f64>,
     ),
@@ -61,7 +62,7 @@ pub fn parse_input(
         })?;
     println!("centerline: data was in plane:{:?} aabb:{:?}", plane, aabb);
 
-    let mut edge_set = fnv::FnvHashSet::<(usize, usize)>::default();
+    let mut edge_set = ahash::AHashSet::<(usize, usize)>::default();
 
     for face in input_pb_model.faces.iter() {
         if face.vertices.len() > 2 {
@@ -141,8 +142,8 @@ pub fn build_output_bp_model(
                 ));
             }
             // unwrap of first and last is safe now that we know there are at least 2 vertices in the list
-            let v0 = super::xy_to_3d(&linestring.points().first().unwrap());
-            let v1 = super::xy_to_3d(&linestring.points().last().unwrap());
+            let v0 = super::xy_to_3d(linestring.points().first().unwrap());
+            let v1 = super::xy_to_3d(linestring.points().last().unwrap());
             let v0_key = transmute_to_u64(&v0);
             let v0_index = *v_map.entry(v0_key).or_insert_with(|| {
                 let new_index = output_pb_model_vertices.len();
@@ -240,7 +241,7 @@ pub fn build_output_bp_model(
             // unwrap of first and last is safe now that we know there are at least 2 vertices in the list
             let v0 = linestring.points().first().unwrap();
             let v1 = linestring.points().last().unwrap();
-            let v0_key = transmute_to_u64(&v0);
+            let v0_key = transmute_to_u64(v0);
             let v0_index = *v_map.entry(v0_key).or_insert_with(|| {
                 let new_index = output_pb_model_vertices.len();
                 output_pb_model_vertices
@@ -248,7 +249,7 @@ pub fn build_output_bp_model(
                 //println!("ls0 pushed ({},{},{})", v0.x, v0.y, v0.z);
                 new_index
             });
-            let v1_key = transmute_to_u64(&v1);
+            let v1_key = transmute_to_u64(v1);
             let v1_index = *v_map.entry(v1_key).or_insert_with(|| {
                 let new_index = output_pb_model_vertices.len();
                 output_pb_model_vertices
@@ -293,6 +294,14 @@ pub fn command(
     a_command: &PB_Command,
     options: HashMap<String, String>,
 ) -> Result<PB_Reply, TBError> {
+    println!(
+        r#"_________                __               .__  .__
+\_   ___ \  ____   _____/  |_  ___________|  | |__| ____   ____
+/    \  \/_/ __ \ /    \   __\/ __ \_  __ \  | |  |/    \_/ __ \
+\     \___\  ___/|   |  \  | \  ___/|  | \/  |_|  |   |  \  ___/
+ \______  /\___  >___|  /__|  \___  >__|  |____/__|___|  /\___  >
+        \/     \/     \/          \/                   \/     \/ "#
+    );
     // angle is supposed to be in degrees
     let cmd_arg_angle = {
         let value = options
@@ -336,7 +345,7 @@ pub fn command(
         )));
     }
     let cmd_arg_max_voronoi_dimension = {
-        let tmp_value = super::MAX_VORONOI_DIMENSION.to_string();
+        let tmp_value = super::DEFAULT_MAX_VORONOI_DIMENSION.to_string();
         let value = options.get("MAX_VORONOI_DIMENSION").unwrap_or(&tmp_value);
         value.parse::<f64>().map_err(|_| {
             TBError::InvalidInputData(format!(
@@ -345,10 +354,12 @@ pub fn command(
             ))
         })?
     };
-    if !(super::MAX_VORONOI_DIMENSION..100_000_000.0).contains(&cmd_arg_max_voronoi_dimension) {
+    if !(super::DEFAULT_MAX_VORONOI_DIMENSION..100_000_000.0)
+        .contains(&cmd_arg_max_voronoi_dimension)
+    {
         return Err(TBError::InvalidInputData(format!(
             "The valid range of MAX_VORONOI_DIMENSION is [{}..100_000_000[% :({})",
-            super::MAX_VORONOI_DIMENSION,
+            super::DEFAULT_MAX_VORONOI_DIMENSION,
             cmd_arg_max_voronoi_dimension
         )));
     }
@@ -455,18 +466,18 @@ pub fn command(
     let shapes = raw_data
         .into_par_iter()
         .map(|shape| {
-            let mut segments = Vec::<boostvoronoi::Line<i64>>::with_capacity(
+            let mut segments = Vec::<geometry::Line<i64>>::with_capacity(
                 shape.set().iter().map(|x| x.len()).sum(),
             );
             for lines in shape.set().iter() {
                 for lineseq in lines.as_lines_iter() {
-                    segments.push(boostvoronoi::Line::new(
+                    segments.push(geometry::Line::new(
                         // boost voronoi only accepts integers as coordinates
-                        boostvoronoi::Point {
+                        geometry::Point {
                             x: lineseq.start.x as i64,
                             y: lineseq.start.y as i64,
                         },
-                        boostvoronoi::Point {
+                        geometry::Point {
                             x: lineseq.end.x as i64,
                             y: lineseq.end.y as i64,
                         },
@@ -516,7 +527,7 @@ pub fn command(
         >>()?;
     //println!("<-build_voronoi");
 
-    let model = build_output_bp_model(&a_command, shapes, cmd_arg_weld, inverted_transform)?;
+    let model = build_output_bp_model(a_command, shapes, cmd_arg_weld, inverted_transform)?;
 
     //println!("<-build_bp_model");
     let mut reply = PB_Reply {
@@ -524,7 +535,8 @@ pub fn command(
             key: "ONLY_EDGES".to_string(),
             value: "True".to_string(),
         }],
-        models: Vec::<PB_Model>::new(),
+        models: Vec::with_capacity(1),
+        models32: Vec::with_capacity(0),
     };
     println!(
         "<-PB_Reply vertices:{:?}, faces:{:?}",
