@@ -13,11 +13,24 @@ use cgmath::{EuclideanSpace, Rad};
 use std::collections::HashMap;
 use std::time;
 
-/// converts to a private, comparable and hash-able format
-/// only use this for floats that are f64::is_finite()
+/// converts to a private, comparable and hash-able format.
+/// Only use this for bit perfect float copies that are f64::is_finite()
 #[inline(always)]
-fn transmute_to_u64(a: &cgmath::Point3<f64>) -> (u64, u64, u64) {
-    (a.x.to_bits(), a.y.to_bits(), a.z.to_bits())
+fn transmute_to_u64(p: &cgmath::Point3<f64>) -> (u64, u64, u64) {
+    let mut x = p.x;
+    let mut y = p.y;
+    let mut z = p.z;
+
+    if x == -0.0 {
+        x = 0.0;
+    }
+    if y == -0.0 {
+        y = 0.0;
+    }
+    if z == -0.0 {
+        z = 0.0;
+    }
+    (x.to_bits(), y.to_bits(), z.to_bits())
 }
 
 /// Algorithmic_botany, page 11 (http://algorithmicbotany.org/papers/#abop)
@@ -32,12 +45,61 @@ fn build_dragon_curve(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>
         .add_axiom("L".to_string())?
         .add_rule('L', "L + R +".to_string())?
         .add_rule('R', "- L - R".to_string())?
+        .round()?
         .exec(cmd_arg_iterations, Turtle::default())?;
 
     println!("build_dragon_curve render() duration: {:?}", now.elapsed());
     Ok(result)
 }
 
+/// https://en.wikipedia.org/wiki/Lévy_C_curve
+fn build_levy_curve(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
+    let now = time::Instant::now();
+
+    let result = TurtleRules::default()
+        .add_token('F', TurtleCommand::Forward(100.0))?
+        .add_token('+', TurtleCommand::Yaw(Rad(45.0f64.to_radians())))?
+        .add_token('-', TurtleCommand::Yaw(Rad(-45.0f64.to_radians())))?
+        .add_axiom("F".to_string())?
+        .add_rule('F', "+ F - - F +".to_string())?
+        .round()?
+        .exec(cmd_arg_iterations, Turtle::default())?;
+
+    println!("build_levy_curve render() duration: {:?}", now.elapsed());
+    Ok(result)
+}
+
+/// build a crooked Lévy C curve in 3d
+fn build_levy_curve_3d(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
+    let now = time::Instant::now();
+
+    let result = TurtleRules::default()
+        .add_token('F', TurtleCommand::Forward(100.0))?
+        .add_token(
+            '+',
+            TurtleCommand::Rotate(
+                Rad(45.0f64.to_radians()),
+                Rad(0.0f64.to_radians()),
+                Rad(0.15f64.to_radians()),
+            ),
+        )?
+        .add_token(
+            '-',
+            TurtleCommand::Rotate(
+                Rad(-45.0f64.to_radians()),
+                Rad(0.0f64.to_radians()),
+                Rad(-0.15f64.to_radians()),
+            ),
+        )?
+        .add_axiom("F".to_string())?
+        .add_rule('F', "+ F - - F +".to_string())?
+        .round()?
+        .exec(cmd_arg_iterations, Turtle::default())?;
+
+    println!("build_levy_curve_3d render() duration: {:?}", now.elapsed());
+    Ok(result)
+}
+
 fn build_dragon_curve_3d(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
     let now = time::Instant::now();
 
@@ -49,6 +111,7 @@ fn build_dragon_curve_3d(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f
         .add_axiom("L".to_string())?
         .add_rule('L', "L + R +".to_string())?
         .add_rule('R', "- L - R".to_string())?
+        .round()?
         .exec(cmd_arg_iterations, Turtle::default())?;
 
     println!(
@@ -59,7 +122,9 @@ fn build_dragon_curve_3d(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f
 }
 
 /// https://en.wikipedia.org/wiki/L-system#Examples_of_L-systems
-fn sierpinski_triangle(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
+fn build_sierpinski_triangle(
+    cmd_arg_iterations: u8,
+) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
     let now = time::Instant::now();
     let result = TurtleRules::default()
         .add_token('F', TurtleCommand::Forward(1.0))?
@@ -70,12 +135,17 @@ fn sierpinski_triangle(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64
         .add_rule('F', " F-G+F+G-F".to_string())?
         .add_rule('G', " GG".to_string())?
         .exec(cmd_arg_iterations, Turtle::default())?;
-    println!("sierpinski_triangle render() duration: {:?}", now.elapsed());
+    println!(
+        "build_sierpinski_triangle render() duration: {:?}",
+        now.elapsed()
+    );
     Ok(result)
 }
 
 /// Algorithmic_botany, page 11 (http://algorithmicbotany.org/papers/#abop)
-fn sierpinski_gasket(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
+fn build_sierpinski_gasket(
+    cmd_arg_iterations: u8,
+) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
     let now = time::Instant::now();
     let result = TurtleRules::default()
         .add_token('R', TurtleCommand::Forward(200.0))?
@@ -86,11 +156,16 @@ fn sierpinski_gasket(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>;
         .add_rule('R', " L - R - L".to_string())?
         .add_rule('L', " R + L + R".to_string())?
         .exec(cmd_arg_iterations, Turtle::default())?;
-    println!("sierpinski_gasket render() duration: {:?}", now.elapsed());
+    println!(
+        "build_sierpinski_gasket render() duration: {:?}",
+        now.elapsed()
+    );
     Ok(result)
 }
 
-fn sierpinski_gasket_3d(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
+fn build_sierpinski_gasket_3d(
+    cmd_arg_iterations: u8,
+) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
     let now = time::Instant::now();
     let result = TurtleRules::default()
         .add_token('R', TurtleCommand::Forward(200.0))?
@@ -101,12 +176,15 @@ fn sierpinski_gasket_3d(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f6
         .add_rule('R', " L - R - L".to_string())?
         .add_rule('L', " R + L + R".to_string())?
         .exec(cmd_arg_iterations, Turtle::default())?;
-    println!("sierpinski_gasket render() duration: {:?}", now.elapsed());
+    println!(
+        "build_sierpinski_gasket render() duration: {:?}",
+        now.elapsed()
+    );
     Ok(result)
 }
 
 /// Algorithmic_botany, page 12 (http://algorithmicbotany.org/papers/#abop)
-fn gosper_curve(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
+fn build_gosper_curve(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
     let now = time::Instant::now();
     let result = TurtleRules::default()
         .add_token('R', TurtleCommand::Forward(1.0))?
@@ -117,11 +195,14 @@ fn gosper_curve(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>,
         .add_rule('L', " L+R++R-L--LL-R+".to_string())?
         .add_rule('R', " -L+RR++R+L--L-R".to_string())?
         .exec(cmd_arg_iterations, Turtle::default())?;
-    println!("sierpinski_gasket render() duration: {:?}", now.elapsed());
+    println!(
+        "build_sierpinski_gasket render() duration: {:?}",
+        now.elapsed()
+    );
     Ok(result)
 }
 
-fn gosper_curve_3d(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
+fn build_gosper_curve_3d(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
     let now = time::Instant::now();
     let result = TurtleRules::default()
         .add_token('R', TurtleCommand::Forward(1.0))?
@@ -134,12 +215,15 @@ fn gosper_curve_3d(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2
         .add_rule('L', " L+R++R-L--LL-R+P".to_string())?
         .add_rule('R', " -L+RR++R+L--L-Rp".to_string())?
         .exec(cmd_arg_iterations, Turtle::default())?;
-    println!("sierpinski_gasket render() duration: {:?}", now.elapsed());
+    println!(
+        "build_sierpinski_gasket render() duration: {:?}",
+        now.elapsed()
+    );
     Ok(result)
 }
 
 /// Algorithmic_botany, page 9 (http://algorithmicbotany.org/papers/#abop)
-fn koch_curve(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
+fn build_koch_curve(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
     let now = time::Instant::now();
     let result = TurtleRules::default()
         .add_token('F', TurtleCommand::Forward(30.0))?
@@ -148,11 +232,11 @@ fn koch_curve(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, T
         .add_axiom("F".to_string())?
         .add_rule('F', " F + F - F - F + F".to_string())?
         .exec(cmd_arg_iterations, Turtle::default())?;
-    println!("koch_curve render() duration: {:?}", now.elapsed());
+    println!("build_koch_curve render() duration: {:?}", now.elapsed());
     Ok(result)
 }
 
-fn koch_curve_3d(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
+fn build_koch_curve_3d(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
     let now = time::Instant::now();
     let result = TurtleRules::default()
         .add_token('F', TurtleCommand::Forward(30.0))?
@@ -161,12 +245,12 @@ fn koch_curve_3d(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>
         .add_axiom("F".to_string())?
         .add_rule('F', " F + F - F - F + F".to_string())?
         .exec(cmd_arg_iterations, Turtle::default())?;
-    println!("koch_curve_3d render() duration: {:?}", now.elapsed());
+    println!("build_koch_curve_3d render() duration: {:?}", now.elapsed());
     Ok(result)
 }
 
 /// Algorithmic_botany, page 9 (http://algorithmicbotany.org/papers/#abop)
-fn quadratic_koch_curve_island(
+fn build_quadratic_koch_curve_island(
     cmd_arg_iterations: u8,
 ) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
     let now = time::Instant::now();
@@ -176,15 +260,16 @@ fn quadratic_koch_curve_island(
         .add_token('-', TurtleCommand::Yaw(Rad(-90.0f64.to_radians())))?
         .add_axiom("F-F-F-F".to_string())?
         .add_rule('F', " F+FF-FF-F-F+F+FF-F-F+F+FF+FF-F".to_string())?
+        .round()?
         .exec(cmd_arg_iterations, Turtle::default())?;
     println!(
-        "quadratic_koch_curve_island render() duration: {:?}",
+        "build_quadratic_koch_curve_island render() duration: {:?}",
         now.elapsed()
     );
     Ok(result)
 }
 
-fn quadratic_koch_curve_island_3d(
+fn build_quadratic_koch_curve_island_3d(
     cmd_arg_iterations: u8,
 ) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
     let now = time::Instant::now();
@@ -198,14 +283,16 @@ fn quadratic_koch_curve_island_3d(
         .add_rule('F', " F+FRFr-FRFr-F-F+F+FRFr-F-F+F+FRFr+FRFr-F".to_string())?
         .exec(cmd_arg_iterations, Turtle::default())?;
     println!(
-        "quadratic_koch_curve_island render() duration: {:?}",
+        "build_quadratic_koch_curve_island render() duration: {:?}",
         now.elapsed()
     );
     Ok(result)
 }
 
 /// https://en.wikipedia.org/wiki/L-system#Examples_of_L-systems
-fn fractal_binary_tree(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
+fn build_fractal_binary_tree(
+    cmd_arg_iterations: u8,
+) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
     let now = time::Instant::now();
     let result = TurtleRules::default()
         .add_token('0', TurtleCommand::Forward(50.0))?
@@ -219,12 +306,15 @@ fn fractal_binary_tree(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64
         .add_rule('0', " 1[L0]R0".to_string())?
         .rotate(Rad(90.0f64.to_radians()), Rad(0.0), Rad(0.0))?
         .exec(cmd_arg_iterations, Turtle::default())?;
-    println!("fractal_binary_tree render() duration: {:?}", now.elapsed());
+    println!(
+        "build_fractal_binary_tree render() duration: {:?}",
+        now.elapsed()
+    );
 
     Ok(result)
 }
 
-fn fractal_binary_tree_3d(
+fn build_fractal_binary_tree_3d(
     cmd_arg_iterations: u8,
 ) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
     let now = time::Instant::now();
@@ -242,7 +332,10 @@ fn fractal_binary_tree_3d(
         .add_rule('0', " 1[L)0]R(0".to_string())?
         .rotate(Rad(90.0f64.to_radians()), Rad(0.0), Rad(0.0))?
         .exec(cmd_arg_iterations, Turtle::default())?;
-    println!("fractal_binary_tree render() duration: {:?}", now.elapsed());
+    println!(
+        "build_fractal_binary_tree_3d render() duration: {:?}",
+        now.elapsed()
+    );
 
     Ok(result)
 }
@@ -250,7 +343,7 @@ fn fractal_binary_tree_3d(
 /// Rather messy implementation of a text parser for a custom turtle builder.
 /// It takes a lot of shortcuts, all parenthesis are ignored, it assumes there is one command per
 /// line etc. etc.
-fn custom_turtle(
+fn build_custom_turtle(
     cmd_arg_iterations: u8,
     cmd_custom_turtle: Option<&String>,
 ) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
@@ -265,13 +358,13 @@ fn custom_turtle(
     let result = TurtleRules::default()
         .parse(cmd_custom_turtle)?
         .exec(cmd_arg_iterations, Turtle::default())?;
-    println!("custom_turtle render() duration: {:?}", now.elapsed());
+    println!("build_custom_turtle render() duration: {:?}", now.elapsed());
 
     Ok(result)
 }
 
 /// https://en.wikipedia.org/wiki/L-system#Examples_of_L-systems
-fn fractal_plant(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
+fn build_fractal_plant(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
     let now = time::Instant::now();
     let result = TurtleRules::default()
         .add_token('X', TurtleCommand::Nop)?
@@ -291,12 +384,17 @@ fn fractal_plant(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>
         .rotate(Rad(70.0f64.to_radians()), Rad(0.0), Rad(0.0))?
         .exec(cmd_arg_iterations, Turtle::default())?;
 
-    println!("fractal_binary_tree render() duration: {:?}", now.elapsed());
+    println!(
+        "build_fractal_binary_tree render() duration: {:?}",
+        now.elapsed()
+    );
     Ok(result)
 }
 
 /// Algorithmic_botany, page 20 (http://algorithmicbotany.org/papers/#abop)
-fn hilbert_curve_3d(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
+fn build_hilbert_curve_3d(
+    cmd_arg_iterations: u8,
+) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
     let now = time::Instant::now();
     let result = TurtleRules::default()
         .add_token('A', TurtleCommand::Nop)?
@@ -318,7 +416,10 @@ fn hilbert_curve_3d(cmd_arg_iterations: u8) -> Result<Vec<[cgmath::Point3<f64>;
         .add_rule('D', " |CFB-F+B|FA&F∧A&&FB-F+B|FC//".to_string())?
         .exec(cmd_arg_iterations, Turtle::default())?;
 
-    println!("fractal_binary_tree render() duration: {:?}", now.elapsed());
+    println!(
+        "build_fractal_binary_tree_3d render() duration: {:?}",
+        now.elapsed()
+    );
     Ok(result)
 }
 
@@ -328,7 +429,12 @@ fn build_output_bp_model(
     _a_command: &PB_Command,
     lines: Vec<[cgmath::Point3<f64>; 2]>,
 ) -> Result<PB_Model, TBError> {
+    // vertex de-duplication mechanism
+    // todo: this is broken since each vertex isn't rounded to int anymore
     let mut vertices_map = ahash::AHashMap::<(u64, u64, u64), usize>::new();
+    // make sure to not add the same edge twice, set key is sorted (lower,higher)
+    let mut dup_edges = ahash::AHashSet::<(usize, usize)>::new();
+
     // Default capacity is probably a little bit too low
     let mut pb_vertices = Vec::<PB_Vertex>::with_capacity(lines.len());
     let mut pb_faces = Vec::<PB_Face>::with_capacity(lines.len());
@@ -361,9 +467,23 @@ fn build_output_bp_model(
             n
         });
 
-        pb_faces.push(PB_Face {
-            vertices: vec![i0 as u64, i1 as u64],
-        });
+        //println!("p0: {:?} is {}", p0, i0);
+        //println!("p1: {:?} is {}", p1, i1);
+
+        let key = {
+            if i0 < i1 {
+                (i0, i1)
+            } else {
+                (i1, i0)
+            }
+        };
+        if !dup_edges.contains(&key) {
+            pb_faces.push(PB_Face {
+                vertices: vec![i0 as u64, i1 as u64],
+            });
+        } else {
+            let _ = dup_edges.insert(key);
+        }
     }
     if !pb_vertices.is_empty() {
         let to_center =
@@ -389,7 +509,7 @@ fn build_output_bp_model(
     }
 
     Ok(PB_Model {
-        name: "Dragon Curve".to_string(),
+        name: "LSystems".to_string(),
         world_orientation: None,
         vertices: pb_vertices,
         faces: pb_faces,
@@ -400,7 +520,14 @@ pub fn command(
     a_command: &PB_Command,
     options: HashMap<String, String>,
 ) -> Result<PB_Reply, TBError> {
-    println!("L Systems got command: \"{}\"", a_command.command);
+    println!(
+        r#".____       _________               __
+|    |     /   _____/__.__. _______/  |_  ____   _____   ______
+|    |     \_____  <   |  |/  ___/\   __\/ __ \ /     \ /  ___/
+|    |___  /        \___  |\___ \  |  | \  ___/|  Y Y  \\___ \
+|_______ \/_______  / ____/____  > |__|  \___  >__|_|  /____  >
+        \/        \/\/         \/            \/      \/     \/ "#
+    );
     if !a_command.models.is_empty() {
         return Err(TBError::InvalidInputData(format!(
             "This operation does not use any models as input:{}",
@@ -412,7 +539,10 @@ pub fn command(
         .ok_or_else(|| TBError::InvalidInputData("Missing the ITERATIONS parameter".to_string()))?
         .parse::<u8>()
         .map_err(|_| {
-            TBError::InvalidInputData("Could not parse the ITERATIONS parameter".to_string())
+            TBError::InvalidInputData(format!(
+                "Could not parse the ITERATIONS parameter: '{}'",
+                options.get("ITERATIONS").unwrap()
+            ))
         })?;
 
     if !(1..21).contains(&cmd_arg_iterations) {
@@ -439,27 +569,29 @@ pub fn command(
     let mesh = match cmd_arg_variant.as_str() {
         "DRAGON_CURVE" => build_dragon_curve(cmd_arg_iterations),
         "DRAGON_CURVE_3D" => build_dragon_curve_3d(cmd_arg_iterations),
-        "FRACTAL_BINARY_TREE" => fractal_binary_tree(cmd_arg_iterations),
-        "FRACTAL_BINARY_TREE_3D" => fractal_binary_tree_3d(cmd_arg_iterations),
-        "FRACTAL_PLANT" => fractal_plant(cmd_arg_iterations),
-        "SIERPINSKI_TRIANGLE" => sierpinski_triangle(cmd_arg_iterations),
-        "SIERPINSKI_GASKET" => sierpinski_gasket(cmd_arg_iterations),
-        "SIERPINSKI_GASKET_3D" => sierpinski_gasket_3d(cmd_arg_iterations),
-        "KOCH_CURVE" => koch_curve(cmd_arg_iterations),
-        "KOCH_CURVE_3D" => koch_curve_3d(cmd_arg_iterations),
-        "GOSPER_CURVE" => gosper_curve(cmd_arg_iterations),
-        "GOSPER_CURVE_3D" => gosper_curve_3d(cmd_arg_iterations),
-        "KOCH_CURVE_ISLAND" => quadratic_koch_curve_island(cmd_arg_iterations),
-        "KOCH_CURVE_ISLAND_3D" => quadratic_koch_curve_island_3d(cmd_arg_iterations),
-        "CUSTOM_TURTLE" => custom_turtle(cmd_arg_iterations, cmd_custom_turtle),
-        "HILBERT_CURVE_3D" => hilbert_curve_3d(cmd_arg_iterations),
+        "FRACTAL_BINARY_TREE" => build_fractal_binary_tree(cmd_arg_iterations),
+        "FRACTAL_BINARY_TREE_3D" => build_fractal_binary_tree_3d(cmd_arg_iterations),
+        "FRACTAL_PLANT" => build_fractal_plant(cmd_arg_iterations),
+        "SIERPINSKI_TRIANGLE" => build_sierpinski_triangle(cmd_arg_iterations),
+        "SIERPINSKI_GASKET" => build_sierpinski_gasket(cmd_arg_iterations),
+        "SIERPINSKI_GASKET_3D" => build_sierpinski_gasket_3d(cmd_arg_iterations),
+        "KOCH_CURVE" => build_koch_curve(cmd_arg_iterations),
+        "KOCH_CURVE_3D" => build_koch_curve_3d(cmd_arg_iterations),
+        "GOSPER_CURVE" => build_gosper_curve(cmd_arg_iterations),
+        "GOSPER_CURVE_3D" => build_gosper_curve_3d(cmd_arg_iterations),
+        "KOCH_CURVE_ISLAND" => build_quadratic_koch_curve_island(cmd_arg_iterations),
+        "KOCH_CURVE_ISLAND_3D" => build_quadratic_koch_curve_island_3d(cmd_arg_iterations),
+        "CUSTOM_TURTLE" => build_custom_turtle(cmd_arg_iterations, cmd_custom_turtle),
+        "HILBERT_CURVE_3D" => build_hilbert_curve_3d(cmd_arg_iterations),
+        "LEVY_C_CURVE" => build_levy_curve(cmd_arg_iterations),
+        "LEVY_C_CURVE_3D" => build_levy_curve_3d(cmd_arg_iterations),
         _ => Err(TBError::InvalidInputData(format!(
             "Invalid CMD_VARIANT parameter:{}",
             cmd_arg_variant
         ))),
     }?;
 
-    let pb_model = build_output_bp_model(&a_command, mesh)?;
+    let pb_model = build_output_bp_model(a_command, mesh)?;
     println!("pb_model.vertices.len() {}", pb_model.vertices.len());
     println!("pb_model.faces.len() {}", pb_model.faces.len());
 
@@ -469,6 +601,7 @@ pub fn command(
             value: "True".to_string(),
         }],
         models: vec![pb_model],
+        models32: Vec::with_capacity(0),
     };
     Ok(reply)
 }
