@@ -415,7 +415,14 @@ pub fn command(
     a_command: &PB_Command,
     options: HashMap<String, String>,
 ) -> Result<PB_Reply, TBError> {
-    println!("simplify got command: \"{}\"", a_command.command);
+    println!(
+        r#"  _________.__               .__  .__  _____
+ /   _____/|__| _____ ______ |  | |__|/ ____\__.__.
+ \_____  \ |  |/     \\____ \|  | |  \   __<   |  |
+ /        \|  |  Y Y  \  |_> >  |_|  ||  |  \___  |
+/_______  /|__|__|_|  /   __/|____/__||__|  / ____|
+        \/          \/|__|                  \/     "#
+    );
     if a_command.models.len() > 1 {
         return Err(TBError::InvalidInputData(format!(
             "This operation only supports one model as input:{}",
@@ -448,14 +455,15 @@ pub fn command(
 
     let lines = find_linestrings(&a_command.models[0])?;
     let lines = simplify_rdp_percent(&a_command.models[0], distance, lines)?;
-    let model = build_bp_model(&a_command, lines)?;
+    let model = build_bp_model(a_command, lines)?;
 
     let mut reply = PB_Reply {
         options: vec![PB_KeyValuePair {
             key: "ONLY_EDGES".to_string(),
             value: "True".to_string(),
         }],
-        models: Vec::<PB_Model>::new(),
+        models: Vec::with_capacity(1),
+        models32: Vec::with_capacity(0),
     };
 
     reply.models.push(model);
