@@ -5,6 +5,7 @@ use logos::Logos;
 use std::f64;
 
 #[derive(Debug, Clone)]
+/// A structure defining the 3D heading of a turtle
 struct Heading {
     heading: Vector3<f64>,
     up: Vector3<f64>,
@@ -20,54 +21,37 @@ impl Default for Heading {
 }
 
 impl Heading {
-    #[allow(dead_code)]
-    fn new(forward: Vector3<f64>, up: Vector3<f64>) -> Self {
-        Self {
-            heading: forward,
-            up,
-        }
-    }
-
-    // rotate around 'forward' or longitudinal axis
+    /// rotate around 'forward' or longitudinal axis
     fn roll(&self, angle: Rad<f64>) -> Heading {
         let rot: Basis3<f64> = Rotation3::from_axis_angle(self.heading, angle);
-        //let rv =
         Self {
             heading: self.heading,
             up: rot.rotate_vector(self.up).normalize(),
         }
-        //println!("Roll angle:{}rad heading:{:?}->{:?}  up:{:?}->{:?}", angle.0, self.heading, rv.heading, self.up, rv.up);
-        //rv
     }
 
-    // rotate around 'up' or vertical axis
+    /// rotate around 'up' or vertical axis
     fn yaw(&self, angle: Rad<f64>) -> Heading {
         let rot: Basis3<f64> = Rotation3::from_axis_angle(self.up, angle);
-        //let rv =
         Self {
             heading: rot.rotate_vector(self.heading).normalize(),
             up: self.up,
         }
-        //println!("Yaw angle:{}rad heading:{:?}->{:?}  up:{:?}->{:?}", angle.0, self.heading, rv.heading, self.up, rv.up);
-        //rv
     }
 
-    // rotate around axis perpendicular to 'up' and 'forward' - i.e. lateral/traverse axis
+    /// rotate around axis perpendicular to 'up' and 'forward' - i.e. lateral/traverse axis
     fn pitch(&self, angle: Rad<f64>) -> Heading {
         let pitch_v = self.heading.cross(self.up).normalize();
         let rot: Basis3<f64> = Rotation3::from_axis_angle(pitch_v, angle);
-        //let rv =
         Self {
             heading: rot.rotate_vector(self.heading).normalize(),
             up: rot.rotate_vector(self.up).normalize(),
         }
-        //println!("Pitch angle:{}rad heading:{:?}->{:?}  up:{:?}->{:?}", angle.0, self.heading, rv.heading, self.up, rv.up);
-        //rv
     }
 
-    // roll, yaw and pitch in order
-    fn rotate(&self, roll: Rad<f64>, yaw: Rad<f64>, pitch: Rad<f64>) -> Heading {
-        self.roll(roll).yaw(yaw).pitch(pitch)
+    /// Perform yaw, pitch and roll in that order
+    fn rotate(&self, yaw: Rad<f64>, pitch: Rad<f64>, roll: Rad<f64>) -> Heading {
+        self.yaw(yaw).pitch(pitch).roll(roll)
     }
 }
 
@@ -76,7 +60,10 @@ pub struct Turtle {
     position: cgmath::Point3<f64>,
     stack: Vec<(Heading, cgmath::Point3<f64>)>,
     result: Vec<[cgmath::Point3<f64>; 2]>,
+    /// Should the turtle draw while moving?
     pen_up: bool,
+    /// should coordinates be rounded to int after each move?
+    round: bool,
 }
 
 impl Default for Turtle {
@@ -87,11 +74,13 @@ impl Default for Turtle {
             result: Vec::new(),
             stack: Vec::new(),
             pen_up: false,
+            round: false,
         }
     }
 }
 
 impl Turtle {
+    /// Apply a turtle command
     fn apply(&mut self, action: &TurtleCommand) -> Result<(), TBError> {
         match action {
             TurtleCommand::Nop => {}
@@ -104,6 +93,12 @@ impl Turtle {
             TurtleCommand::Forward(distance) => {
                 let l1 = self.position;
                 self.position += self.heading.heading * *distance;
+                if self.round {
+                    self.position.x = self.position.x.round();
+                    self.position.y = self.position.y.round();
+                    self.position.z = self.position.z.round();
+                }
+
                 //println!("Self.heading: {:?}, Self.heading*distance={:?}", self.heading.heading, self.heading.heading * *distance);
                 if !self.pen_up {
                     self.result.push([l1, self.position]);
@@ -148,6 +143,7 @@ pub struct TurtleRules {
     yaw: Option<Rad<f64>>,
     pitch: Option<Rad<f64>>,
     roll: Option<Rad<f64>>,
+    round: bool,
 }
 
 impl TurtleRules {
@@ -213,6 +209,12 @@ impl TurtleRules {
         Ok(self)
     }
 
+    /// Make sure that the turtle round()s the coordinate after each move.
+    pub fn round(&mut self) -> Result<&mut Self, TBError> {
+        self.round = true;
+        Ok(self)
+    }
+
     /// Expands the rules over the axiom 'n' times
     fn expand(&self, n: u8) -> Result<Vec<char>, TBError> {
         let mut rv: Vec<char> = self.axiom.chars().collect();
@@ -221,12 +223,12 @@ impl TurtleRules {
             for v in rv.iter() {
                 if v == &' ' {
                     continue;
-                } else if let Some(rule) = self.rules.get(&v) {
+                } else if let Some(rule) = self.rules.get(v) {
                     // it was a rule
                     tmp.append(&mut rule.chars().collect());
                 } else {
                     // maybe a token?
-                    let _ = self.tokens.get(&v).ok_or_else(|| {
+                    let _ = self.tokens.get(v).ok_or_else(|| {
                         eprintln!("tokens: {:?}", self.tokens.keys());
                         eprintln!("rules: {:?}", self.rules.keys());
                         TBError::LSystems3D(format!("Could not find rule or token:'{}'", &v))
@@ -278,6 +280,9 @@ impl TurtleRules {
             #[regex("\\.?roll")]
             Roll,
 
+            #[regex("\\.?round")]
+            Round,
+
             #[token("Turtle::Forward")]
             TurtleActionForward,
 
@@ -497,6 +502,11 @@ impl TurtleRules {
                         )));
                     }
                 },
+                ParseToken::Round => {
+                    println!("Got .round()");
+                    self.round = true;
+                    state = ParseState::Start;
+                }
                 ParseToken::EOL => {
                     line += 1;
                     state = ParseState::Start;
@@ -581,6 +591,9 @@ impl TurtleRules {
     ) -> Result<Vec<[cgmath::Point3<f64>; 2]>, TBError> {
         let path = self.expand(n)?;
 
+        if self.round {
+            turtle.round = true;
+        }
         // Apply initial rotations
         if let Some(yaw) = self.yaw {
             turtle.apply(&TurtleCommand::Yaw(yaw))?;
