@@ -1,15 +1,26 @@
-#![feature(hash_drain_filter)]
-#![deny(non_camel_case_types)]
-#![deny(unused_parens)]
-#![deny(non_upper_case_globals)]
-#![deny(unused_qualifications)]
-#![deny(unused_results)]
-#![deny(unused_imports)]
+#![deny(
+    rust_2018_compatibility,
+    rust_2018_idioms,
+    nonstandard_style,
+    unused,
+    future_incompatible,
+    non_camel_case_types,
+    unused_parens,
+    non_upper_case_globals,
+    unused_qualifications,
+    unused_results,
+    unused_imports,
+    unused_variables
+)]
+
+#![cfg_attr(feature = "hash_drain_filter", feature(hash_drain_filter))]
+#![cfg_attr(feature = "map_first_last", feature(map_first_last))]
 
 mod cmd_2d_outline;
 mod cmd_centerline;
 mod cmd_knife_intersect;
 mod cmd_lsystems;
+mod cmd_sdf;
 mod cmd_simplify;
 mod cmd_voronoi;
 mod cmd_voronoi_mesh;
@@ -19,7 +30,6 @@ mod voronoi_utils;
 
 use crate::toxicblend_pb::Command as PB_Command;
 use crate::toxicblend_pb::KeyValuePair as PB_KeyValuePair;
-use crate::toxicblend_pb::Model as PB_Model;
 use crate::toxicblend_pb::Reply as PB_Reply;
 use crate::toxicblend_pb::Vertex as PB_Vertex;
 use cgmath::Zero;
@@ -34,11 +44,13 @@ use toxicblend_pb::toxic_blend_service_server::{ToxicBlendService, ToxicBlendSer
 const EPSILON: f64 = 0.0001;
 
 /// The largest dimension of the voronoi input, totally arbitrarily selected.
-const MAX_VORONOI_DIMENSION: f64 = 256.0 * 800.0;
+const DEFAULT_MAX_VORONOI_DIMENSION: f64 = 200000.0;
 
-/// The length of one 'step' for curved edges discretization
-const VORONOI_DISCRETE_DISTANCE: f64 = 0.0001;
+/// The length of one 'step' for curved edges discretization as a percentage of the longest
+/// AABB axis of the object.
+const DEFAULT_VORONOI_DISCRETE_DISTANCE: f64 = 0.0001;
 
+#[allow(unused_qualifications)] // tonic codegen has unused_qualifications
 pub mod toxicblend_pb {
     tonic::include_proto!("toxicblend");
 }
@@ -63,7 +75,7 @@ impl PB_Vertex {
     }
 }
 
-/// converts a Point2 to Point3 using the XY coordinates
+/// converts a Point2 to Point3 using the XY coordinates, sets Z to zero.
 #[inline(always)]
 pub fn xy_to_3d(point: &cgmath::Point2<f64>) -> cgmath::Point3<f64> {
     cgmath::Point3 {
@@ -83,6 +95,9 @@ fn xy_to_2d(point: &cgmath::Point3<f64>) -> cgmath::Point2<f64> {
 
 #[derive(thiserror::Error, Debug)]
 pub enum TBError {
+    #[error("Overflow error: {0}")]
+    Overflow(String),
+
     #[error("Unknown command received: {0}")]
     UnknownCommand(String),
 
@@ -119,6 +134,9 @@ pub enum TBError {
     #[error(transparent)]
     BoostVoronoiError(#[from] boostvoronoi::BvError),
 
+    #[error(transparent)]
+    JoinError(#[from] tokio::task::JoinError),
+
     #[error("Error in LSystems {0}")]
     LSystems3D(String),
 }
@@ -126,6 +144,7 @@ pub enum TBError {
 #[derive(Debug, Default)]
 pub struct TheToxicBlendService {}
 
+#[inline(always)]
 /// convert the options to a hashmap
 fn options_to_map(options: &[PB_KeyValuePair]) -> HashMap<String, String> {
     let mut rv = HashMap::<String, String>::new();
@@ -142,20 +161,18 @@ impl ToxicBlendService for TheToxicBlendService {
         request: PB_Request<PB_Command>,
     ) -> Result<PB_Response<PB_Reply>, PB_Status> {
         let a_command = request.get_ref();
-        let map = options_to_map(&a_command.options);
-        println!("########################################");
-        println!("# Received command:{}", a_command.command.as_str());
-        println!("########################################");
+        let options_map = options_to_map(&a_command.options);
 
         let rv = match a_command.command.as_str() {
-            "2d_outline" => cmd_2d_outline::command(a_command, map),
-            "simplify" => cmd_simplify::command(a_command, map),
-            "knife_intersect" => cmd_knife_intersect::command(a_command, map),
-            "centerline" => cmd_centerline::command(a_command, map),
-            "voronoi_mesh" => cmd_voronoi_mesh::command(a_command, map),
-            "voronoi" => cmd_voronoi::command(a_command, map),
-            "voxel" => cmd_voxel::command(a_command, map),
-            "lsystems" => cmd_lsystems::command(a_command, map),
+            "2d_outline" => cmd_2d_outline::command(a_command, options_map),
+            "simplify" => cmd_simplify::command(a_command, options_map),
+            "knife_intersect" => cmd_knife_intersect::command(a_command, options_map),
+            "centerline" => cmd_centerline::command(a_command, options_map),
+            "voronoi_mesh" => cmd_voronoi_mesh::command(a_command, options_map),
+            "voronoi" => cmd_voronoi::command(a_command, options_map),
+            "voxel" => cmd_voxel::command(a_command, options_map),
+            "lsystems" => cmd_lsystems::command(a_command, options_map),
+            "sdf" => cmd_sdf::command(a_command, options_map),
             _ => Err(TBError::UnknownCommand(a_command.command.clone())),
         };
         // convert TBError to a valid Error message
@@ -166,9 +183,11 @@ impl ToxicBlendService for TheToxicBlendService {
                     key: "ERROR".to_string(),
                     value: format!("{:?}", err),
                 }],
-                models: Vec::<PB_Model>::with_capacity(0),
+                models: Vec::with_capacity(0),
+                models32: Vec::with_capacity(0),
             }))
         } else {
+            // unwrap is safe now
             Ok(PB_Response::new(rv.unwrap()))
         }
     }
